<?php

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if( !class_exists( 'WC_Gateway_Pay_on_pickup' ) ):

/**
 * Main plugin class
 *
 * @usedby WC_Payment_Gateway
 */
class WC_Gateway_Pay_on_pickup extends WC_Payment_Gateway {

    /**
     * Init languages files and gateway settigns
     */
    public function __construct() {
        load_plugin_textdomain( 'wpd-wooconnect', false, dirname( dirname( plugin_basename( __FILE__ ) ) ) . '/i18n/' );

        $this->id                = 'pop';
        $this->icon              = apply_filters('woocommerce_pop_icon', '');
        $this->has_fields        = false;
        $this->method_title      = __( 'Pay on pickup', 'wpd-wooconnect' );
        $this->order_button_text = apply_filters( 'woocommerce_pop_order_button_text', __( 'Place order', 'wpd-wooconnect' ) );

        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();

        // Define user set variables
        // Get settings
        $this->title              = $this->get_option( 'title' );
        $this->description        = $this->get_option( 'description' );
        $this->instructions       = $this->get_option( 'instructions' );
        $this->enable_for_methods = $this->get_option( 'enable_for_methods', array() );

        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
        add_action( 'woocommerce_thankyou_pop', array( $this, 'thankyou' ) );
    }

    /**
     * Admin Panel Options
     * - Options for bits like 'title' and availability on a country-by-country basis
     *
     * @access public
     * @return void
     */
    function admin_options() {
        ?>
        <h3><?php _e('Pay on Pickup','wpd-wooconnect'); ?></h3>
        <p><?php _e('Have your customers pay with cash (or by other means) on pickup.', 'wpd-wooconnect' ); ?></p>
        <table class="form-table">
            <?php $this->generate_settings_html(); ?>
        </table>
    <?php
    }

    /**
     * Create form fields for the payment gateway
     *
     * @return void
     */
    public function init_form_fields() {
        $shipping_methods = array();

        if ( is_admin() ) {
            foreach ( WC()->shipping->load_shipping_methods() as $method ) {
                $shipping_methods[ $method->id ] = $method->get_title();
            }
        }

        $this->form_fields = array(
            'enabled' => array(
                'title' => __( 'Enable/Disable', 'wpd-wooconnect' ),
                'type' => 'checkbox',
                'label' => __( 'Enable Pay On Pickup', 'wpd-wooconnect' ),
                'default' => 'yes'
            ),
            'title' => array(
                'title' => __( 'Title', 'wpd-wooconnect' ),
                'type' => 'text',
                'description' => __( 'This controls the title which the user sees during checkout', 'wpd-wooconnect' ),
                'default' => __( 'Pay on pickup', 'wpd-wooconnect' ),
                'desc_tip'      => true,
            ),
            'description' => array(
                'title' => __( 'Customer Message', 'wpd-wooconnect' ),
                'type' => 'textarea',
                'default' => __( 'Pay your order in cash as you pick it up at our dispensary.', 'wpd-wooconnect' )
            ),
            'instructions' => array(
                'title' => __( 'Instructions', 'wpd-wooconnect' ),
                'type' => 'textarea',
                'description' => __( 'Instructions that will be added to the thank you page.', 'wpd-wooconnect' ),
                'default' => __( 'Pay with cash on pickup at [Store address].', 'wpd-wooconnect' )
            ),
            'enable_for_methods' => array(
                'title'         => __( 'Enable for shipping methods', 'wpd-wooconnect' ),
                'type'          => 'multiselect',
                'class'         => 'chosen_select',
                'css'           => 'width: 450px;',
                'default'       => '',
                'description'   => __( 'If POP is only available for certain methods, set it up here. Leave blank to enable for all methods.', 'wpd-wooconnect' ),
                'options'       => $shipping_methods,
                'desc_tip'      => true,
            )
        );
    }

    /**
     * Check If The Gateway Is Available For Use
     *
     * @return bool
     */
    public function is_available() {
        if ( ! empty( $this->enable_for_methods ) ) {

            $chosen_shipping_methods_session = WC()->session->get( 'chosen_shipping_methods' );

            if ( isset( $chosen_shipping_methods_session ) ) {
                $chosen_shipping_methods = array_unique( $chosen_shipping_methods_session );
            } else {
                $chosen_shipping_methods = array();
            }

            $check_method = false;

            if ( is_page( wc_get_page_id( 'checkout' ) ) && ! empty( $wp->query_vars['order-pay'] ) ) {

                $order_id = absint( $wp->query_vars['order-pay'] );
                $order    = new WC_Order( $order_id );

                if ( $order->shipping_method )
                    $check_method = $order->shipping_method;

            } elseif ( empty( $chosen_shipping_methods ) || sizeof( $chosen_shipping_methods ) > 1 ) {
                $check_method = false;
            } elseif ( sizeof( $chosen_shipping_methods ) == 1 ) {
                $check_method = $chosen_shipping_methods[0];
            }

            if ( ! $check_method )
                return false;

            $found = false;

            foreach ( $this->enable_for_methods as $method_id ) {
                if ( strpos( $check_method, $method_id ) === 0 ) {
                    $found = true;
                    break;
                }
            }

            if ( ! $found )
                return false;
        }

        return parent::is_available();
    }

    /**
     * Process the order payment status
     *
     * @param int $order_id
     * @return array
     */
    public function process_payment( $order_id ) {
        $order = new WC_Order( $order_id );

        // Mark as on-hold (we're awaiting the cheque)
        $order->update_status( apply_filters( 'wc_pop_default_order_status', 'on-hold' ), __( 'Awaiting cash payment', 'wpd-wooconnect' ) );

        // Reduce stock levels
        $order->reduce_order_stock();

        // Remove cart
        WC()->cart->empty_cart();

        // Return thankyou redirect
        return array(
            'result'    => 'success',
            'redirect'  => $this->get_return_url( $order )
        );
    }

    /**
     * Output for the order received page.
     *
     * @return void
     */
    public function thankyou() {
        echo $this->instructions != '' ? wpautop( wptexturize( wp_kses_post( $this->instructions ) ) ) : '';
    }
}
endif;
